<?php 

/*
Plugin Name: Dave's External Search
Description: Search against Flickr, YouTube, and other sources
Author: Dave Ross
Version: 0.1
Author URI: http://davidmichaelross.com
*/

include_once(ABSPATH . WPINC . '/rss-functions.php');

if(!class_exists('DavesFileCache'))
	include('DavesFileCache.php');

if(5.0 > floatval(phpversion()))
	die("Dave's External Search requires PHP 5.0 or higher");

class DavesExternalSearch extends WP_Widget
{
  const FLICKR_API_KEY = '8e847ed53de31295a781b1741c87f54d';
  const FLICKR_API_SECRET = 'f94c8ea2bbbbd475';
  
	private $alertMessages;
	
	/////////
	// Setup
	/////////
	
	public static function register()
	{
		register_widget(__CLASS__);	
	}
	  
	// Constructor
	function DavesExternalSearch() {
		$this->alertMessages = array();
		
		$widget_ops = array ('classname' => 'daves_external_search', 'description' => __('Search external sites') );
		$control_ops = null;
		
		$this->WP_Widget('daves_external_search', __("Dave's External Search"), $widget_ops, $control_ops);
	}
	
	/////////////////////////////
	// WP_Widget implementation
	/////////////////////////////
	
	function widget($args, $instance)
	{
		if(is_search())
		{
			extract($args, EXTR_SKIP);
	 
	 		// Before Widget
			echo $before_widget;
			
			// Title
			$title = empty($instance['title']) ? '&nbsp;' : apply_filters('widget_title', $instance['title']);
			if ( !empty( $title ) ) { echo $before_title . $title . $after_title; };
	
			// Flickr Results
			$results = self::flickrCall(self::FLICKR_API_KEY, 'flickr.photos.search', array(
				'user_id' => $instance['flickr_nsid'],
				'text' => $_GET['s'],
				'per_page' => 6
			));
			
			$photos = $results['photos']['photo'];
			if($photos)
			{
				echo "<h4>".__("Results from Flickr")."</h4>";
			}
			
			foreach($photos as $index=>$photo)
			{
				$url = self::flickrPhotoUrl($photo);
				echo "<a href=\"http://www.flickr.com/photos/{$photo['owner']}/{$photo['id']}\"><img src=\"$url\" title=\"{$photo['title']}\" style=\"margin: 5px;\" /></a>";
			}
			
			// YouTube Results
			$feedURL = self::youTubeURL($instance['youtube_username'], $_GET['s']);
			try {
				 $cache = DavesFileCache::forIdentifier($feedURL);
				 $feed = $cache->get();
				 print "<!-- YOUTUBE FROM CACHE!!! -->";	
			}
			catch(Exception $e) {
				$feed = fetch_rss($feedURL);
				$cache = new DavesFileCache($feedURL);
				$cache->store($feed, 300);
			}
			
			if($feed->items)
			{
				echo "<h4>".__("Results from YouTube")."</h4>";
			
				foreach($feed->items as $item)
				{
					$id = substr($item['id'], strrpos($item['id'] , '/')+1);
					echo self::youTubeEmbed($id, 150, 100);
				}
			}
						
			// Twitter Results
			$twitterSearchURL = self::twitterSearchURL($instance['twitter_username'], $_GET['s']);
			try {
				 $cache = DavesFileCache::forIdentifier($twitterSearchURL);
				 $feed = $cache->get();
				 print "<!-- TWITTER FROM CACHE!!! -->";	
			}
			catch(Exception $e) {
				$feed = fetch_rss($twitterSearchURL);
				$cache = new DavesFileCache($twitterSearchURL);
				$cache->store($feed, 300);
			}
			
			if($feed->items)
			{
				echo "<h4>".__("Results from Twitter")."</h4>";
				
				foreach($feed->items as $item)
				{
					$publishedTime = date('M d, Y h:i:sa', strtotime($item['published']));
					
					echo "{$item['title']} :: {$item['author_name']} at <a href=\"{$item['link']}\">$publishedTime</a><br /><br />";
				}
			}
		
			// After Widget
			echo $after_widget;
		}
	}
	
	function update($new_instance, $old_instance)
	{
		$instance = $old_instance;

		$instance['title'] = strip_tags($new_instance['title']);
		$instance['flickr_username'] = strip_tags($new_instance['flickr_username']);
		$instance['twitter_username'] = strip_tags($new_instance['twitter_username']);
		$instance['youtube_username'] = strip_tags($new_instance['youtube_username']);
		
		if($old_instance['flickr_username'] != $new_instance['flickr_username'])
		{
			// Flickr username has changed. Fetch the new username's NSID
			$results = self::flickrCall(self::FLICKR_API_KEY, 'flickr.people.findByUsername', array('username' => $new_instance['flickr_username']));

			if('fail' == $results['stat'])
			{
				$this->alertMessages['flickr_username'] = $instance['flickr_username'].': '.$results['message'];
				return FALSE;
			}

			$instance['flickr_nsid'] = $results['user']['nsid'];
		}
		
		return $instance;
	}
	
	function form ($instance)
	{
		$instance = wp_parse_args( (array) $instance, array('title' => '', 'text' => '', 'flickr_username' => '', 'twitter_username' => '', 'youtube_username' => ''));
		
		$title = strip_tags($instance['title']);
		$text = format_to_edit($instance['text']);
		$flickr_username = format_to_edit($instance['flickr_username']);
		$twitter_username = format_to_edit($instance['twitter_username']);
		$youtube_username = format_to_edit($instance['youtube_username']);
		
		?>
		
		<p>
		<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?>
		<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo attribute_escape($title); ?>" />
		</label>
		</p>
		
		<p>
		<label for="<?php echo $this->get_field_id('flickr_username'); ?>"><?php _e('Flickr Username:'); ?>
		<input class="widefat" id="<?php echo $this->get_field_id('flickr_username'); ?>" name="<?php echo $this->get_field_name('flickr_username'); ?>" type="text" value="<?php echo attribute_escape($flickr_username); ?>" />
		</label>
		<?php if($this->alertMessages['flickr_username']) : ?>
		<?php echo $this->alertMessages['flickr_username']; ?>
		<?php endif; ?>
		</p>
		
		<p>
		<label for="<?php echo $this->get_field_id('twitter_username'); ?>"><?php _e('Twitter Username:'); ?>
		<input class="widefat" id="<?php echo $this->get_field_id('twitter_username'); ?>" name="<?php echo $this->get_field_name('twitter_username'); ?>" type="text" value="<?php echo attribute_escape($twitter_username); ?>" />
		</label>
		</p>
		
		<p>
		<label for="<?php echo $this->get_field_id('youtube_username'); ?>"><?php _e('YouTube Username:'); ?>
		<input class="widefat" id="<?php echo $this->get_field_id('youtube_username'); ?>" name="<?php echo $this->get_field_name('youtube_username'); ?>" type="text" value="<?php echo attribute_escape($youtube_username); ?>" />
		</label>
		</p>
		
		<?php
	}
	
	////////////////////
	// Flickr Interface
	////////////////////
	function flickrCall($apiKey, $method, $params)
	{
		// Build URL
		$url = "http://api.flickr.com/services/rest/?method=$method&format=php_serial&api_key=$apiKey";
		
		// Append parameters
		foreach($params as $name=>$value)
		{
			$url .= "&{$name}={$value}";
		}
		
		// Submit & return response
		try {
			 $cache = DavesFileCache::forIdentifier($url);
			 $result = $cache->get();
			 print "<!-- FLICKR FROM CACHE!!! -->";	
		}
		catch(Exception $e) {
			$result = file_get_contents($url);
			$cache = new DavesFileCache($url);
			$cache->store($result, 300);
		}
		$result = unserialize($result);

		return $result;
	}
	
	function flickrPhotoUrl($photo)
	{
		extract($photo);
		return self::flickrURL($farm, $server, $id, $secret, 's');	
	}
	
	function flickrURL($farmID, $serverID, $id, $secret, $size = 's')
	{
		$url = "http://farm{$farmID}.static.flickr.com/{$serverID}/{$id}_{$secret}_$size.jpg";
		
		return $url;
	}

	/////////////////////	
	// YouTube Interface
	/////////////////////
	function youTubeUrl($userName, $query)
	{
		// Build URL
		$query = urlencode($query);
		$url = "http://gdata.youtube.com/feeds/api/users/$userName/uploads?q=$query";
		return $url;
	}
	
	function youTubeEmbed($id, $width = 320, $height = 265, $language = 'en')
	{
		$embed = <<<EMBED_CODE
	<object width="{$width}" height="{$height}"><param name="movie" value="http://www.youtube.com/v/{$id}&hl={$language}&fs=1&"></param><param name="allowFullScreen" value="true"></param><param name="allowscriptaccess" value="always"></param><embed src="http://www.youtube.com/v/{$id}&hl={$language}&fs=1&" type="application/x-shockwave-flash" allowscriptaccess="always" allowfullscreen="true" width="{$width}" height="{$height}"></embed></object>	
EMBED_CODE;

		return $embed;
	}
	
	/////////////////////
	// Twitter Interface
	/////////////////////
	function twitterSearchURL($userName, $query)
	{
		$userName = urlencode($userName);
		$query = urlencode($query);
		
		return "http://search.twitter.com/search.atom?q=$query%20from%3A$userName";
	}
}

// Register
add_action('widgets_init', array('DavesExternalSearch', 'register'));

?>